clear all
close all
format compact
disp(' ');
disp('----------------------------');
% tester les formules d'incertitude avec des simulations numriques
%% generate random numbers
% compare stixbox algorithm and matlab statistics toolbox algo
m = 100; % mean
s = 15; % std deviation
nb = 1000; % combien de nombres alatoires? 

% stixbox version
tic;
V1 = randn(nb,1).*s + m;
stix = toc;
disp(['randn time = ' num2str(stix) ' sec.'])
% cette fonction utilise simplement une fonction matlab
% randn(nb).*s + m;

% statistics toolbox
tic;
V2 = normrnd(m,s,nb,1);
toolbx = toc;
disp(['normrnd time = ' num2str(toolbx) ' sec.'])
disp(['toolbx/stix = ' num2str(toolbx/stix)])

bins = 50;
hiMin = min(min(V1),min(V2));
hiMax = max(max(V1),max(V2));
limits = linspace(hiMin,hiMax,bins);
V1Hist = histc(V1,limits);
V2Hist = histc(V2,limits);
plot(limits,V1Hist,'.',limits,V2Hist,'d');

% CONCLUSION:  both versions give (probably) identical results but the 
% toolbox is much slower than the basic version.

%% Test of std error formula
% Real distribution
% If we made an infinite number of measurements with our instrument we
% would receive this distribution characterized by RealMean and RealSigma.
RealMean = 100;
RealSigma = 20;
bins = 100;
limits = linspace(0,200,bins);
figure(2);
RealPDF = normpdf(limits,RealMean,RealSigma);
plot(limits,RealPDF);

% Measurement and analysis
nbMeas = 10;
V1 = randn(nbMeas,1).*RealSigma + RealMean;
V2 = normrnd(RealMean,RealSigma,nbMeas,1);
disp(' '),
disp(['nbMeas = ' num2str(nbMeas)]);
V1Chars = CharacS(V1);
disp(['V1: ' num2str(V1Chars(2)) '  ' ...
    num2str(V1Chars(3)) '  ' num2str(V1Chars(4))]);
V2Chars = CharacS(V2);
disp(['V2: ' num2str(V2Chars(2)) '  ' ...
    num2str(V2Chars(3)) '  ' num2str(V2Chars(4))]);
figure(2);
hold on
plot(V1Chars,max(RealPDF)*0.9,'dr',V1Chars,max(RealPDF)*0.9,'-');
hold on
plot(V2Chars,max(RealPDF)*0.8,'dk');

% Is this really the 95.44997% confidence interval?
repetitions = 1000; % repeat the measurement above many times
MeanS = zeros(repetitions,1);
UpLim = zeros(repetitions,1);
LowLim = zeros(repetitions,1);
for repNo = 1:repetitions
    V = randn(nbMeas,1).*RealSigma + RealMean;
    % V = normrnd(RealMean,RealSigma,nbMeas,1);
    VChars = CharacS(V);
    MeanS(repNo) = VChars(3);
    UpLim(repNo) = VChars(4);
    LowLim(repNo) = VChars(2);
end
outside = length( UpLim((UpLim < RealMean) | (LowLim > RealMean)) );
inside = length( UpLim((UpLim > RealMean) & (LowLim < RealMean)) );
disp(' ');
disp(['Total = ' num2str(outside+inside)]);
disp(['Outside = ' num2str(outside/(outside+inside)*100) ' %']);
disp(['Theor. outside = ' num2str(100-95.44997) ' %']);

% With student-t correction
% tinv(0.975,50) is the factor for 95% confidence and 51 samples
tCorrErrbar = (UpLim - MeanS)./2 .* tinv( 1-(1.00-0.9544997)/2 , nbMeas-1 );
tUpLim = MeanS + tCorrErrbar;
tLowLim = MeanS - tCorrErrbar;
toutside = length( tUpLim((tUpLim < RealMean) | (tLowLim > RealMean)) );
tinside = length( tUpLim((tUpLim > RealMean) & (tLowLim < RealMean)) );
disp(['Outside with t-correction = ' num2str(toutside/(toutside+tinside)*100) ' %']);

% Did I understand what 'bootsrapping' means?
% make a statistics on the simulation results
disp(' ');
disp(['Mean of 95.4% err-bars = ' num2str(mean(UpLim-MeanS))]);
disp(['Mean of t-corrected err-bars = ' num2str(mean(tCorrErrbar))]);
% figure(3)
% hist(MeanS);
MeanS = sort(MeanS);
limIndx = ceil(repetitions*(1.00-0.9544997)/2);
disp(['Bootstrapping (ceil) = ' num2str(mean([ MeanS(end-limIndx)-mean(MeanS),...
    mean(MeanS)-MeanS(limIndx)]))]);
limIndx = floor(repetitions*(1.00-0.9544997)/2);
disp(['Bootstrapping (floor) = ' num2str(mean([ MeanS(end-limIndx)-mean(MeanS),...
    mean(MeanS)-MeanS(limIndx)]))]);

% CONCLUSION: Everything as expected 
% - except the mean of the t-corrected error-bars

%% make a graphs as function of sample size

nbMeasS = [3 5 10 15 20 50 100 300 1000];
% nbMeasS = [10 100];
repetitions = 1000; % repeat the measurement above many times

% memory alocation
% the means of these values over the repetitions
errBootsStrappCeil = zeros(size(nbMeasS));
errBootsStrappFloor = zeros(size(nbMeasS));
err2Sigma = zeros(size(nbMeasS));
err_tCorr = zeros(size(nbMeasS)); 
stdDevS = zeros(size(nbMeasS));  
% the ratio of samples for which the realMean is outide of the interval
Outside2Sigma = zeros(size(nbMeasS));
Outside_tCorr = zeros(size(nbMeasS));

for nbMeasCount = 1:length(nbMeasS)
    nbMeas = nbMeasS(nbMeasCount);
    
    % calculation
    MeanS = zeros(repetitions,1);
    UpLim = zeros(repetitions,1);
    LowLim = zeros(repetitions,1);
    stdV = zeros(repetitions,1);
    for repNo = 1:repetitions
        V = randn(nbMeas,1).*RealSigma + RealMean;
        % V = normrnd(RealMean,RealSigma,nbMeas,1);
        VChars = CharacS(V);
        MeanS(repNo) = VChars(3);
        UpLim(repNo) = VChars(4);
        LowLim(repNo) = VChars(2);
        stdV(repNo) = std(V);
    end
    outside = length( UpLim((UpLim < RealMean) | (LowLim > RealMean)) );
    
    % tCorrErrbar = (UpLim - MeanS)./2 .* tinv( 1-(1.00-0.9544997)/2 , nbMeas-1 );
    tCorrErrbar = stdV./sqrt(nbMeas).* tinv( 1-(1.00-0.9544997)/2 , nbMeas-1 );
    tUpLim = MeanS + tCorrErrbar;
    tLowLim = MeanS - tCorrErrbar;
    toutside = length( tUpLim((tUpLim < RealMean) | (tLowLim > RealMean)) );
    
    % output
    % the means of these values over the repetitions
    err2Sigma(nbMeasCount) = mean(UpLim-MeanS);
    err_tCorr(nbMeasCount) = mean(tCorrErrbar); 
    stdDevS(nbMeasCount) = mean(stdV);
    % the ratio of samples for which the realMean is outide of the interval
    Outside2Sigma(nbMeasCount) = outside/repetitions*100;
    Outside_tCorr(nbMeasCount) = toutside/repetitions*100;
    
    % the Bootstapping results
    MeanS = sort(MeanS);
    limIndx = ceil(repetitions*(1.00-0.9544997)/2);
    errBootsStrappCeil(nbMeasCount) = mean([ MeanS(end-limIndx+1)-mean(MeanS),...
                                               mean(MeanS)-MeanS(limIndx)]);
    limIndx = floor(repetitions*(1.00-0.9544997)/2);
    errBootsStrappFloor(nbMeasCount) = mean([ MeanS(end-limIndx)-mean(MeanS),...
                                               mean(MeanS)-MeanS(limIndx+1)]);
end

figure(10);
semilogx(nbMeasS,Outside2Sigma,'.k',nbMeasS,Outside_tCorr,'*r',...
                    nbMeasS,(100-95.44997).*ones(size(nbMeasS)),'-b');
title('% of repetitions where RealMean is outside conf. interval');
legend('Outside 2Sigma', 'Outside tCorr', 'alpha theo');
xlabel('sample size');
ylim([0 max(Outside2Sigma)+1]);

figure(11);
semilogx(nbMeasS,2*stdDevS,'-k',nbMeasS,err2Sigma,'.k',...
                    nbMeasS,err_tCorr,'*r', ...
      nbMeasS,errBootsStrappCeil,'--g', nbMeasS,errBootsStrappFloor,'--g');
title('% means of the obtained uncertainties (95.4% confidence)');
legend('2*stdDevS', 'err2Sigma', 'err tCorr', 'errBootsStrapp');
xlabel('sample size');


%% weighted average - one run - (combining series with unequal sample size)
nbMeasS = [3 5 10 50 100];

% generation of measurement series with different nb of samples per series
% memory allocation 
FullV = zeros(1, sum(nbMeasS)); 
DiffMeanS = zeros(1, length(nbMeasS)); 
DiffStdS = zeros(1, length(nbMeasS)); 

for nbMeasCount = 1:length(nbMeasS)
    nbMeas = nbMeasS(nbMeasCount);
    V = randn(nbMeas,1).*RealSigma + RealMean;
    % V = normrnd(RealMean,RealSigma,nbMeas,1);
    DiffMeanS(nbMeasCount) = mean(V);
    DiffStdS(nbMeasCount) = std(V);
    if nbMeasCount == 1
        fullV(1:nbMeasS(nbMeasCount)) = V;
    else
        fullV( sum(nbMeasS(1:nbMeasCount-1))+1 : sum(nbMeasS(1:nbMeasCount))) = V;
    end
end

% 1) One global mean
disp(' ');
FullMean = mean(fullV);
FullStdErr = std(fullV)/sqrt(length(fullV));
disp(['Global mean of all measurements = ' num2str(FullMean)]);
disp(['Std error for this value = ' num2str(FullStdErr)]);

% 2) Using intermediate means and making a weighted average
% 2a) Weight = nbMeasS (if nbMeas is known for each sub-mean)
wi = nbMeasS;  
WeiNbMean = (wi * DiffMeanS.')/sum(wi); % wi and DiffMeanS need to be lines
% WeiNbMeanStdErr = 1/sqrt(sum(wi)); % result of calculation using error propagation and 1/StdErr^2 weights
disp(' ');
disp(['Mean of means weighted with sub-N = ' num2str(WeiNbMean)]);
% disp(['Std error for this value = ' num2str(WeiNbMeanStdErr)]);
disp(['Deviation from ideal value = ' num2str((WeiNbMean-FullMean)/FullMean*100) ' %']);
% CONCLUSION: This is the ideal weight, result identical to global mean.
% ??? CHECK FOR THE ERROR for comparison with global case !!!  ???

% 2b) Weight = 1/(sub-std-err)^2
wi = 1./(DiffStdS ./ sqrt(nbMeasS)).^2;  
WeiStdErrMean = (wi * DiffMeanS.')/sum(wi); % wi and DiffMeanS need to be lines
WeiStdErrMeanStdErr = 1/sqrt(sum(wi)); % result of calculation using error propagation and 1/StdErr^2 weights
disp(' ');
disp(['Mean of means weighted with 1/StdErr^2 = ' num2str(WeiStdErrMean)]);
disp(['Std error for this value = ' num2str(WeiStdErrMeanStdErr)]);
disp(['Deviation from ideal value = ' num2str((WeiStdErrMean-FullMean)/FullMean*100) ' %']);
% CONCLUSION: This sounds resonable, but result not identical to global
% mean and the error is smaller than for the global mean.

% 2c) Weight = 1/(t-corr-sub-std-err)^2
wi = 1./( DiffStdS ./ sqrt(nbMeasS) .* tinv( 1-(1.00-0.68269)/2 , nbMeasS-1 ) ).^2;  
WeiTcorrStdErrMean = (wi * DiffMeanS.')/sum(wi); % wi and DiffMeanS need to be lines
% WeiTcorrStdErrMeanStdErr = 1/sqrt(sum(wi)); % result of calculation using error propagation and 1/StdErr^2 weights
disp(' ');
disp(['Mean of means weighted with 1/(t-Corr-StdErr)^2 = ' num2str(WeiTcorrStdErrMean)]);
% disp(['Std error for this value = ' num2str(WeiStdErrMeanStdErr)]);
disp(['Deviation from ideal value = ' num2str((WeiTcorrStdErrMean-FullMean)/FullMean*100) ' %']);
% CONCLUSION: With respect to the version without t-correction, the result comes 
% closer to the global mean (but it's not identical) 
% ??? BEST CADIDATE - CALCULATE THE ERROR !!!  ???

% 2c95) Weight = 1/(t-corr95-sub-std-err)^2
wi = 1./( DiffStdS ./ sqrt(nbMeasS) .* tinv( 1-(1.00-0.95)/2 , nbMeasS-1 ) ).^2;  
WeiTcorr95StdErrMean = (wi * DiffMeanS.')/sum(wi); % wi and DiffMeanS need to be lines
% WeiTcorr95StdErrMeanStdErr = 1/sqrt(sum(wi)); % result of calculation using error propagation and 1/StdErr^2 weights
disp(' ');
disp(['Mean of means weighted with 1/(t-Corr95-StdErr)^2 = ' num2str(WeiTcorr95StdErrMean)]);
% disp(['Std error for this value = ' num2str(WeiStdErrMeanStdErr)]);
disp(['Deviation from ideal value = ' num2str((WeiTcorr95StdErrMean-FullMean)/FullMean*100) ' %']);
% CONCLUSION: even better ? need to make averages over many repetitions

% 2d) Weight = 1/(sub-StdDev)^2
wi = 1./DiffStdS.^2; 
WeiStdDevMean = (wi * DiffMeanS.')/sum(wi); % wi and DiffMeanS need to be lines
% WeiStdDevMeanStdErr = 1/sqrt(sum(wi)); % result of calculation using error propagation and 1/StdErr^2 weights
disp(' ');
disp(['Mean of means weighted with 1/StdErr^2 = ' num2str(WeiStdDevMean)]);
% disp(['Std error for this value = ' num2str(WeiStdDevMeanStdErr)]);
disp(['Deviation from ideal value = ' num2str((WeiStdDevMean-FullMean)/FullMean*100) ' %']);
% CONCLUSION: As the std dev increases with the number of measurements as to reach 
% the standard deviation of the instrument (see fig 11) -> THIS IS RUBBISH 
% (A precise sub-measurement does not have a large weight, but a small one). 
% It's also the version that gives the largest deviation from tha global
% mean

% 2e) Weight = 1 (No weight)
wi = ones(size(DiffMeanS)); 
WeiNoMean = (wi * DiffMeanS.')/sum(wi); % wi and DiffMeanS need to be lines
% WeiNoMeanStdErr = 1/sqrt(sum(wi)); % result of calculation using error propagation and 1/StdErr^2 weights
disp(' ');
disp(['Mean of means NOT weighted = ' num2str(WeiNoMean)]);
% disp(['Std error for this value = ' num2str(WeiStdDevMeanStdErr)]);
disp(['Deviation from ideal value = ' num2str((WeiNoMean-FullMean)/FullMean*100) ' %']);
% CONCLUSION: not as bad as weighting with the std-deviation, but still bad

%% weighted average - many repetitions - (combining series with unequal sample size)

repetitions = 200; % repeat the measurement above many times

% memory alocation
% the means of these values over the repetitions
FullMeanS = zeros(1, repetitions);
WeiStdErrMeanS = zeros(1, repetitions);
WeiTcorrStdErrMeanS = zeros(1, repetitions);
WeiTcorr95StdErrMeanS = zeros(1, repetitions);

for repCount = 1:repetitions

end
% 
% figure(10);
% semilogx(nbMeasS,Outside2Sigma,'.k',nbMeasS,Outside_tCorr,'*r',...
%                     nbMeasS,(100-95.44997).*ones(size(nbMeasS)),'-b');
% title('% of repetitions where RealMean is outside conf. interval');
% legend('Outside 2Sigma', 'Outside tCorr', 'alpha theo');
% xlabel('sample size');
% ylim([0 max(Outside2Sigma)+1]);
% 
% figure(11);
% semilogx(nbMeasS,2*stdDevS,'-k',nbMeasS,err2Sigma,'.k',...
%                     nbMeasS,err_tCorr,'*r', ...
%       nbMeasS,errBootsStrappCeil,'--g', nbMeasS,errBootsStrappFloor,'--g');
% title('% means of the obtained uncertainties (95.4% confidence)');
% legend('2*stdDevS', 'err2Sigma', 'err tCorr', 'errBootsStrapp');
% xlabel('sample size');
% 


